unit B2FMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Registry;

type
  // Record for use as list element
  PUrlRec = ^TUrlRec;
  TUrlRec = record
    Rep:     String;         // Folder name (folders and links)
    UrlName: String;         // Logical URL name (links only)
    UrlPath: String;         // www URL name (links only)
    Level: Integer;          // Folder level (position in folder tree)
  end;

type
  TMainForm = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Label1: TLabel;
    Edit1: TEdit;
    Label2: TLabel;
    Edit2: TEdit;
    OpenDialog1: TOpenDialog;
    Button4: TButton;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
  private
    BookmarkList: TStringList;
    L: TList;
    LastFolder: String;
    TargetFolder: String;
    LvlList: TStringList;
    procedure GetFavoritesFolder(var Folder: String);
    procedure Convert(Folder: String);
    procedure ScanBmkLine(I: Integer; Lvl: Integer);
    function FindFolder(HtmlStr: String; var Lvl: Integer): Boolean;
    function FindUrl(HtmlStr: String; Lvl: Integer): Boolean;
    function FindSectionEnd(HtmlStr: String; var Lvl: Integer): Boolean;
    procedure MakeFavorites;
    function MakeFolder(Folder: String): Boolean;
    function MakeUrlFile(UrlName, UrlPath: String): Boolean;
    procedure ReplaceIllChars(var S: String);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  IniFiles, B2FAbout;

{---------------------- TMainForm ---------------------}

procedure TMainForm.GetFavoritesFolder(var Folder: String);
// Find the directory of the Internet Explorer favorites
var
  Registry: TRegistry;
begin
  Registry := TRegistry.Create;
  Registry.RootKey := HKEY_CURRENT_USER;
  Registry.OpenKey('Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', False);
  Folder := Registry.ReadString('Favorites');
  Registry.Free;
end;


function SortFunction(Item1, Item2: Pointer): Integer;
// Function to sort the list of bookmarks alphabetically
var
  Rec1, Rec2: PUrlRec;
begin
  Result := 0;
  Rec1 := Item1;
  Rec2 := Item2;
  if Rec1.Rep < Rec2.Rep then Result := -1;
  if Rec1.Rep > Rec2.Rep then Result := 1;
  if Rec1.Rep = Rec2.Rep then
  begin
    if Rec1.UrlName < Rec2.UrlName then Result := -1;
    if Rec1.UrlName > Rec2.UrlName then Result := 1;
    if Rec1.UrlName = Rec2.UrlName then Result := 0;
  end;
end;


procedure TMainForm.Convert(Folder: String);
var
  I: Integer;
begin
  L := TList.Create;
  LvlList := TStringList.Create;
  LvlList.Add('');
  try
    ScanBmkLine(0, 0);       // Find all bookmarks recursively
  finally
    L.Sort(SortFunction);    // Sort the resulting list
    MakeFavorites;           // Make favorites from the list
    // Clean up
    for I := 0 to L.Count -1 do
      Dispose(PUrlRec(L[I]));
    L.Free;
    LvlList.Free;
  end;
end;


procedure TMainForm.ScanBmkLine(I: Integer; Lvl: Integer);
// Recursive method to scan all lines in the bookmark file
begin
  if I < BookmarkList.Count -1 then
  begin
    if not FindSectionEnd(BookmarkList[I], Lvl) then
      if not FindUrl(BookmarkList[I], Lvl) then
        FindFolder(BookmarkList[I], Lvl);
    ScanBmkLine(I+1, Lvl);
  end;
end;


function TMainForm.FindFolder(HtmlStr: String; var Lvl: Integer): Boolean;
// Retrive folder name (if any) from HTML string
const
  FolderSubStr: String = '<H3';
var
  J, Idx: Integer;
  S: array[0..255] of Char;
  UrlRec: PUrlRec;
  I: Integer;
  Folder: String;
  FName: String;
begin
  J := Pos(FolderSubStr, HtmlStr);
  Result := (J <> 0);
  if J <> 0 then             // Substring found, retrieve folder name
  begin
    Inc(Lvl);                // 1 level up

    FillChar(S, SizeOf(S), 0);
    Idx := 1;
    for J := 1 to Length(HtmlStr)-1 do
      if HtmlStr[J] = '>' then
        Idx := J+1;
    for J := Idx to Length(HtmlStr)-5 do
      S[J-Idx] := HtmlStr[J];

    Folder := S;
    ReplaceIllChars(Folder);

    if LvlList.Count > Lvl then
      LvlList[Lvl] := Folder          // Replace folder hierarchy
    else
      LvlList.Add(Folder);

    FName := LvlList[Lvl];
    for I := Lvl-1 downto 1 do
      FName := LvlList[I] + '\' + FName;
//    FName := FName + '\' + Folder;

    // Finally store a record with the data
    UrlRec := New(PUrlRec);
    UrlRec.Rep := FName;
    UrlRec.UrlName := '';
    UrlRec.UrlPath := '';
    UrlRec.Level := Lvl;
    L.Add(UrlRec);
  end;
end;


function TMainForm.FindUrl(HtmlStr: String; Lvl: Integer): Boolean;
// Retrive URL name and path (if any) from HTML string
const
  UrlSubStr: String = 'http://';
var
  J, K: Integer;
  S1, S2, S3: array[0..255] of Char;
  Apo1, Apo2: PChar;
  I: Integer;
  SPath, SName: String;
  FName: String;
  UrlRec: PUrlRec;
begin
  J := Pos(UrlSubStr, HtmlStr);
  Result := (J <> 0);
  if J <> 0 then             // Substring found, retrieve URL path
  begin
    FillChar(S1, SizeOf(S1), 0);
    FillChar(S2, SizeOf(S2), 0);
    FillChar(S3, SizeOf(S3), 0);
    SPath := HtmlStr;
    Apo1 := StrScan(PChar(SPath), '"');     // Requires " !!!!!
    for K := 1 to StrLen(Apo1) do
      S1[K-1] := Apo1[K];
    for K := 0 to Pos('"', S1) -2 do
      S2[K] := S1[K];

    SPath := S2;
    // SPath should now hold the proper URL path

    // Now retrieve URL name
    Apo2 := StrScan(Apo1, '>');
    for K := 1 to StrLen(Apo2) do
      S1[K-1] := Apo2[K];
    for K := 0 to Pos('<', S1) -2 do
      S3[K] := S1[K];

    // Convert illegal characters
    SName := S3;
    ReplaceIllChars(SName);

    FName := LvlList[Lvl];
    for I := Lvl-1 downto 1 do
      FName := LvlList[I] + '\' + FName;

    // Finally store a record with the data
    UrlRec := New(PUrlRec);
    UrlRec.Rep := FName;
    UrlRec.UrlName:= SName;
    UrlRec.UrlPath := SPath;
    UrlRec.Level := Lvl;
    L.Add(UrlRec);
  end;
end;


function TMainForm.FindSectionEnd(HtmlStr: String; var Lvl: Integer): Boolean;
// Return whether the current section ends
const
  EndSubStr: String = '</DL>';
var
  J, Idx: Integer;
  UrlRec: PUrlRec;
begin
  J := Pos(EndSubStr, HtmlStr);
  Result := (J <> 0);
  if J <> 0 then             // Substring found, end of section
  begin
    Dec(Lvl);                // 1 level down
  end;
end;


procedure TMainForm.MakeFavorites;
// Create the favorites and their folders
var
  I: Integer;
  UrlRec: PUrlRec;
begin
  for I := 0 to L.Count -1 do
  begin
    UrlRec := L[I];
    if UrlRec.UrlName = '' then
      MakeFolder(TargetFolder + '\' + UrlRec.Rep)
    else
      MakeUrlFile(TargetFolder + '\' + UrlRec.Rep + '\' {+IntToStr(UrlRec.Level)+' - '} + UrlRec.UrlName + '.url', UrlRec.UrlPath);
  end;
end;


function TMainForm.MakeFolder(Folder: String): Boolean;
begin
  CreateDirectory(PChar(Folder), nil);
end;


function TMainForm.MakeUrlFile(UrlName, UrlPath: String): Boolean;
var
  UrlFile: TIniFile;
begin
  { A URL-file has the structure of an INI-file with just one section.
    Create the file as an INI-file and save the URL path. }
  UrlFile := TIniFile.Create(UrlName);
  UrlFile.WriteString('InternetShortcut', 'URL', UrlPath);
  UrlFile.Free;
end;


procedure TMainForm.ReplaceIllChars(var S: String);
{ Replace illegal chars in filenames. This is necessary since
  Internet Explorer stores the names as files and folders. }
const
  ReplacedChar: Char = '-';
var
  I: Integer;
begin
  for I := 1 to Length(S) do
    if S[I] in ['\', '/', ':', '*', '?', '"', '<', '>', '|'] then
      S[I] := ReplacedChar;
end;


procedure TMainForm.Button1Click(Sender: TObject);
var
  R: TSearchRec;
  FavFolder: String;         // Path of the favorites folder
begin
  if not FileExists(Edit2.Text) then
  begin
    MessageDlg('The bookmark file you specified does not exist.',
               mtError, [MbOk], 0);
    Edit2.SetFocus;
  end
  else
  begin
    BookmarkList := TStringList.Create;
    // Load the bookmarks into the stringlist
    BookmarkList.LoadFromFile(Edit2.Text);
    GetFavoritesFolder(FavFolder);
    if FavFolder <> '' then
    begin
      // First make the appropriate favorites subfolder
      CreateDirectory(PChar(FavFolder+'\'+Edit1.Text), nil);
      // Make sure the folder actually exists
      if FindFirst(FavFolder+'\'+Edit1.Text, faDirectory, R) = 0 then
      begin
        // Now convert all the bookmark links
        TargetFolder := FavFolder+'\'+Edit1.Text;
        Convert(FavFolder+'\'+Edit1.Text);
        MessageDlg('The bookmarks have been copied to your favorites.',
                   mtInformation, [MbOk], 0);
      end
      else
      begin
        MessageDlg('The subfolder name you specified is invalid.',
                   mtError, [MbOk], 0);
        Edit1.SetFocus;
      end;
      FindClose(R);
    end;
    BookmarkList.Free;
  end;
end;


procedure TMainForm.Button2Click(Sender: TObject);
begin
  Close;
end;


procedure TMainForm.Button3Click(Sender: TObject);
begin
  AboutBox.ShowModal;
end;


procedure TMainForm.Button4Click(Sender: TObject);
begin
  if OpenDialog1.Execute then
    Edit2.Text := OpenDialog1.FileName;
end;

end.

