unit F2BMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Registry;

type
  // Record for use as list element
  PUrlRec = ^TUrlRec;
  TUrlRec = record
    Rep:     String;         // Folder name (folders and links)
    UrlName: String;         // Logical URL name (links only)
    UrlPath: String;         // www URL name (links only)
    Level: Integer;          // Folder level (position in folder tree)
  end;

type
  TMainForm = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Label1: TLabel;
    Edit1: TEdit;
    CheckBox1: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    FavFolder: String;       // Path of the favorites folder
    FavList: TList;          // List of the favorite links
    BmkList: TStringList;    // Favorites converted into bookmarks
    procedure GetFavoritesFolder;
    procedure SearchURL(Folder: String; Level: Integer);
    procedure MakeBookmarkFile;
    procedure TraverseFavList(Idx, PrevIdx: Integer);
  protected
    procedure MakeDocumentTop;
    procedure MakeDocumentBottom;
    procedure MakeHeaderTop(UrlRec: PUrlRec);
    procedure MakeHeaderBottom(UrlRec: PUrlRec);
    procedure MakeBookmark(UrlRec: PUrlRec);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

uses
  IniFiles, ShellApi, F2BAbout;

{---------------------- TMainForm ---------------------}

procedure TMainForm.GetFavoritesFolder;
// Find the directory of the Internet Explorer favorites
var
  Registry: TRegistry;
begin
  Registry := TRegistry.Create;
  Registry.RootKey := HKEY_CURRENT_USER;
  Registry.OpenKey('Software\Microsoft\Windows\CurrentVersion\Explorer\Shell Folders', False);
  FavFolder := Registry.ReadString('Favorites');
  Registry.Free;
end;


procedure TMainForm.SearchURL(Folder: String; Level: Integer);
// Research and create the list of the favorites files found.
// This method uses recursion to get all the subfolders.
var
  Found: Integer;
  SearchRec: TSearchRec;
  UrlFile: TIniFile;
  UrlRec: PUrlRec;
begin
  // First File
  Found := FindFirst(Folder+'\*.*', faAnyFile, SearchRec);
  // Research other file
  while Found = 0 do
  begin
    // Don't get the special directories ('.' and '..')
    if (SearchRec.Name <> '.') and (SearchRec.Name <> '..') then
      // Change directory
      if (SearchRec.Attr and faDirectory > 0) then
        begin
          if not (SearchRec.Attr and faSysFile > 0) then
          begin
            UrlRec := New(PUrlRec);
            UrlRec.Rep := Copy(Folder+'\'+SearchRec.Name, Length(FavFolder)+2, Length(Folder+'\'+SearchRec.Name));
///            UrlRec.Rep := SearchRec.Name;
            UrlRec.UrlName := '';
            UrlRec.UrlPath := '';
            UrlRec.Level := Level;
            FavList.Add(UrlRec);
          end;
          SearchURL(Folder+'\'+SearchRec.Name, Level+1);   // Recursion
        end
      else
        begin
          // Get the file extension
          if UpperCase(ExtractFileExt(SearchRec.Name)) = '.URL' then
          begin
            UrlRec := New(PUrlRec);
            UrlRec.Rep := Copy(Folder, Length(FavFolder)+2, Length(Folder));
            UrlRec.UrlName:= Copy(SearchRec.Name, 0, Length(SearchRec.Name)-3);
            { A URL-file has the structure of an INI-file with just one section.
              Open the file as an INI-file to read the URL path. }
            UrlFile := TIniFile.Create(Folder+'\'+SearchRec.Name);
            UrlRec.UrlPath := UrlFile.ReadString('InternetShortcut', 'URL', 'no path');
            UrlRec.Level := Level;
            UrlFile.Free;
            FavList.Add(UrlRec);       // Add to list of favorites
          end;
        end;
      // other file
      Found := FindNext(SearchRec);
  end;
{
  // Add a dummy record
       UrlRec := New(PUrlRec);
       UrlRec.Rep := '';
       UrlRec.UrlName := '';
       UrlRec.UrlPath := '';
       UrlRec.Level := 0;
       FavList.Add(UrlRec);
}
end;


procedure TMainForm.MakeBookmarkFile;
// Create the bookmark file with the internet favorites files found
begin
  BmkList := TStringList.Create;

  MakeDocumentTop;
  TraverseFavList(0, -1);    // Called recursively
///  MakeHeaderBottom(FavList[FavList.Count-1]);
  MakeDocumentBottom;

  try
    BmkList.SaveToFile(Edit1.Text);
  except on EFCreateError do
    begin
      MessageDlg('The target file name you specified is invalid.', mtError, [MbOk], 0);
      Edit1.SetFocus;
    end;
  end;

  BmkList.Free;
  MessageDlg('Your favorites have been copied to the bookmark file you specified.',
             mtInformation, [MbOk], 0);
end;


procedure TMainForm.TraverseFavList(Idx, PrevIdx: Integer);
// Recursive method to obtain all URLs in their respective folders
var
  UrlRec, PrevUrlRec: PUrlRec;
  X: Integer;
begin
  if Idx < FavList.Count then
  begin

    UrlRec := FavList[Idx];
    if PrevIdx = -1 then
      PrevUrlRec := nil
    else
      PrevUrlRec := FavList[PrevIdx];

    if UrlRec.UrlName = '' then   // Folder found (not a URL)
    begin
      if PrevUrlRec <> nil then
      begin
        X := PrevUrlRec.Level;
        while UrlRec.Level <= X do
        begin
          MakeHeaderBottom(PrevUrlRec);      // End section
          Dec(PrevUrlRec.Level);           // dirty!
          Dec(X);
        end;
      end;

      MakeHeaderTop(UrlRec);      // Make a new header

      PrevIdx := Idx;
    end
    else
      MakeBookmark(UrlRec);       // Insert bookmark

    TraverseFavList(Idx+1, PrevIdx);

  end;
end;


procedure TMainForm.MakeDocumentTop;
begin
  BmkList.Add('<!-- Made with ' + Application.Title + ' -->');
  BmkList.Add('');
  BmkList.Add('<TITLE>' + 'Converted Bookmarks' + '</TITLE>');
  BmkList.Add('<H1>' + 'Converted Bookmarks' + '</H1>');
  BmkList.Add('');
  BmkList.Add('<DL><P>');
end;


procedure TMainForm.MakeDocumentBottom;
begin
///  BmkList.Add('');
//  BmkList.Add('    </DL><P>');
  BmkList.Add('');
  BmkList.Add('</DL><P>');
end;


procedure TMainForm.MakeHeaderTop(UrlRec: PUrlRec);
// Add header name to the start of the section
var
  I, Idx: Integer;
  S: String;
  A: array[0..255] of Char;
begin
  // Change names of subfolders
  FillChar(A, SizeOf(A), 0);
  Idx := 1;
  for I := 1 to Length(UrlRec.Rep) do
    if UrlRec.Rep[I] = '\' then
      Idx := I+1;
  for I := Idx to Length(UrlRec.Rep) do
    A[I-Idx] := UrlRec.Rep[I];

  // Add the name to the header
  BmkList.Add('');
///  BmkList.Add('Level: ' + IntToStr(UrlRec.Level));
  for I := 1 to UrlRec.Level do
    S := S + '    ';
  BmkList.Add(S + '<DT><H3>' + A + '</H3>');
  BmkList.Add(S + '<DL><P>');
end;


procedure TMainForm.MakeHeaderBottom(UrlRec: PUrlRec);
// End section containing bookmarks
var
  I: Integer;
  S: String;
begin
  for I := 1 to UrlRec.Level do
    S := S + '    ';
  BmkList.Add(S + '</DL><P>');
  BmkList.Add('');
end;


procedure TMainForm.MakeBookmark(UrlRec: PUrlRec);
var
  I: Integer;
  S: String;
begin
  { For some reason the names of all the favorite links retrieved
    have an extra dot in the end. Remove this dot. }
  Delete(UrlRec.UrlName, Length(UrlRec.UrlName), 1);
  // Now add the path and name
  for I := 1 to UrlRec.Level do
    S := S + '    ';
///  BmkList.Add('Level: ' + IntToStr(UrlRec.Level));
  BmkList.Add(S + '<DT><A HREF="' + UrlRec.UrlPath + '">' +
              UrlRec.UrlName + '</A>' + '<BR>');
end;


function SortFunction(Item1, Item2: Pointer): Integer;
// Function to sort the list of favorites alphabetically
var
  Rec1, Rec2: PUrlRec;
begin
  Result := 0;
  Rec1 := Item1;
  Rec2 := Item2;
  if Rec1.Rep < Rec2.Rep then Result := -1;
  if Rec1.Rep > Rec2.Rep then Result := 1;
  if Rec1.Rep = Rec2.Rep then
  begin
    if Rec1.UrlName < Rec2.UrlName then Result := -1;
    if Rec1.UrlName > Rec2.UrlName then Result := 1;
    if Rec1.UrlName = Rec2.UrlName then Result := 0;
  end;
end;


procedure TMainForm.Button1Click(Sender: TObject);
var
  I: Integer;
begin
  FavList := TList.Create;
  GetFavoritesFolder;
  if FavFolder <> '' then
  begin
    SearchURL(FavFolder, 1);      // Get all links
    FavList.Sort(SortFunction);   // Sort the resulting list
    MakeBookmarkFile;             // Make bookmark file from links
  end;
  // Clean up
  for I := 0 to FavList.Count -1 do
    Dispose(PUrlRec(FavList[I]));
  FavList.Free;
  // Open the new bookmark file if user wants it
  if CheckBox1.Checked then
    if ShellExecute(Application.MainForm.Handle, nil, PChar(Edit1.Text),
                    nil, nil, SW_SHOWNORMAL) <= 32 then
      MessageDlg('Error: Could not start your Internet browser.', mtError, [mbOk], 0);
// Alternatively you could use ExecuteFile
end;


procedure TMainForm.Button2Click(Sender: TObject);
begin
  Close;
end;


procedure TMainForm.Button3Click(Sender: TObject);
begin
  AboutBox.ShowModal;
end;

end.

